#
# Mercenaries Mod
# CvMercEventManager
# 

from CvPythonExtensions import *
import CvUtil

import CvEventManager
import sys
import PyHelpers
import CvMainInterface
import CvMercenaryManager
import MercenaryUtils
import CvScreenEnums 
import CvConfigParser
import Popup as PyPopup
		
gc = CyGlobalContext()	

objMercenaryUtils = MercenaryUtils.MercenaryUtils()

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()
PyInfo = PyHelpers.PyInfo

# Set g_bGameTurnMercenaryCreation to true if mercenary creation should happen during the 
# onBeginGameTurn method, false if it should happen during the onBeginPlayerTurn method
# Default value is true
g_bGameTurnMercenaryCreation = true

# Set g_bDisplayMercenaryManagerOnBeginPlayerTurn to true if the "Mercenary Manager" 
# screen should be displayed at the beginning of every player turn. 
# Default value is false
g_bDisplayMercenaryManagerOnBeginPlayerTurn = false

# This value also controls the "Mercenary Manager" button and when it should be displayed.
# Default value is "ERA_ANCIENT"
g_iStartingEra = gc.getInfoTypeForString("ERA_ANCIENT")

# Change this to false if mercenaries should be removed from the global mercenary pool 
# at the beginning of the game turn. When set to true a number of mercenaries will 
# wander away from the global mercenary pool. This is another variable used to control 
# the load time for the "Mercenary Manager" screen.
# Default valus is true
g_bWanderlustMercenaries = true

# Change this to increase the max number of mercenaries that may wander away from the
# global mercenary pool.
# Default valus is 3
g_iWanderlustMercenariesMaximum = 3

# Change this to false to supress the mercenary messages.
# Default value is true
g_bDisplayMercenaryMessages = true

# Set to true to print out debug messages in the logs
g_bDebug = false

# globals
###################################################
class CvMercEventManager:

	mercenaryManager = None
	
	def __init__(self, eventManager):
	
		self.EventKeyDown=6
		self.eventManager = eventManager

		# initialize base class
		eventManager.addEventHandler("BeginGameTurn", self.onBeginGameTurn)
		eventManager.addEventHandler("BeginPlayerTurn", self.onBeginPlayerTurn)
		eventManager.addEventHandler("kbdEvent",self.onKbdEvent)
		eventManager.addEventHandler("unitLost",self.onUnitLost)
		eventManager.addEventHandler("unitKilled",self.onUnitKilled)
		eventManager.addEventHandler("OnLoad",self.onLoadGame)
		eventManager.addEventHandler("GameStart",self.onGameStart)
		eventManager.addEventHandler("unitPromoted",self.onUnitPromoted)
		

		self.mercenaryManager = CvMercenaryManager.CvMercenaryManager(CvScreenEnums.MERCENARY_MANAGER)		

		global g_bGameTurnMercenaryCreation
		global g_bDisplayMercenaryManagerOnBeginPlayerTurn
		global g_iStartingEra
		global g_bWanderlustMercenaries
		global g_iWanderlustMercenariesMaximum
		global g_bDisplayMercenaryMessages 
		
		# Load the Mercenaries Mod Config INI file containing all of the configuration information		
		config = CvConfigParser.CvConfigParser("Mercenaries Mod Config.ini")
		
		# If we actually were able to open the "Mercenaries Mod Config.ini" file then read in the values.
		# otherwise we'll keep the default values that were set at the top of this file.
		if(config != None):
			g_bGameTurnMercenaryCreation = config.getboolean("Mercenaries Mod", "Game Turn Mercenary Creation", true)
			g_bDisplayMercenaryManagerOnBeginPlayerTurn = config.getboolean("Mercenaries Mod", "Display Mercenary Manager On Begin Player Turn", false)
			g_iStartingEra = gc.getInfoTypeForString(config.get("Mercenaries Mod","Starting Era","ERA_ANCIENT"))
			g_bWanderlustMercenaries = config.getboolean("Mercenaries Mod", "Wanderlust Mercenaries", true)
			g_iWanderlustMercenariesMaximum = config.getint("Mercenaries Mod","Wanderlust Mercenaries Maximum", 5)
			g_bDisplayMercenaryMessages = config.getboolean("Mercenaries Mod", "Display Mercenary Messages", true)

		objMercenaryUtils = MercenaryUtils.MercenaryUtils()

	
	# This method creates a new instance of the MercenaryUtils class to be used later
	def onLoadGame(self, argsList):

		global objMercenaryUtils

		objMercenaryUtils = MercenaryUtils.MercenaryUtils()


	# This method create a new instance of the MercenaryUtils class to be used later
	def onGameStart(self, argsList):
	
		global objMercenaryUtils
		
		objMercenaryUtils = MercenaryUtils.MercenaryUtils()


	# This method will update the players gold, make some mercenaries wander away and add 
	# mercenaries to the global mercenary pool.
	def onBeginGameTurn(self, argsList):
		# Get the list of active players in the game
		playerList = PyGame.getCivPlayerList()
		
		# Go through each of the players and deduct their mercenary maintenance amount from their gold
		for i in range(len(playerList)):
			playerList[i].setGold(playerList[i].getGold()-objMercenaryUtils.getPlayerMercenaryMaintenanceCost(playerList[i].getID()))
			playerList[i].setGold(playerList[i].getGold()+objMercenaryUtils.getPlayerMercenaryContractIncome(playerList[i].getID()))
	
		# Have some mercenaries wander away from the global mercenary pool if 
		# g_bWanderlustMercenaries is set to true.	
		if(g_bWanderlustMercenaries):
			wanderingMercenaryCount = gc.getGame().getMapRand().get(g_iWanderlustMercenariesMaximum, "Random Num")
			objMercenaryUtils.removeMercenariesFromPool(wanderingMercenaryCount)
			
		# Add the mercenaries to the global mercenary pool if the g_bGameTurnMercenaryCreation 
		# is set to true
		if(g_bGameTurnMercenaryCreation):
			objMercenaryUtils.addMercenariesToPool()				
			
	
	# This method will add mercenaries to the global mercenary pool, display the mercenary manager screen
	# and provide the logic to make the computer players think.
	def onBeginPlayerTurn(self, argsList):		
		iGameTurn, iPlayer = argsList

		player = gc.getPlayer(iPlayer)

		# Debug code - start
		if(g_bDebug):
			CvUtil.pyPrint(player.getName() + " Gold: " + str(player.getGold()) + " is human: " + str(player.isHuman()))
		# Debug code - end		
		
		# Add the mercenaries to the global mercenary pool if the 
		# g_bGameTurnMercenaryCreation is set to false
		if(not g_bGameTurnMercenaryCreation):
			objMercenaryUtils.addMercenariesToPool()

		# if g_bDisplayMercenaryManagerOnBeginPlayerTurn is true the the player is human
		# then display the mercenary manager screen
		if(g_bDisplayMercenaryManagerOnBeginPlayerTurn and player.isHuman()):
			self.mercenaryManager.interfaceScreen()

		# if the player is not human then run the think method
		if(not player.isHuman()):
			objMercenaryUtils.computerPlayerThink(iPlayer)
		
		# Place any mercenaries that might be ready to be placed.
		objMercenaryUtils.placeMercenaries(iPlayer)
		
		
	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		
		player = gc.getPlayer(iPlayer)

		CyInterface().addImmediateMessage(player.getName(),"")
			
			
	# This method handles the key input and will bring up the mercenary manager screen if the 
	# player has at least one city and presses the 'M' key.
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'
		# TO DO: REMOVE THE FOLLOWING LINE BEFORE RELEASE.
		#gc.getPlayer(0).setGold(20000)
		eventType,key,mx,my,px,py = argsList
			
		theKey=int(key)

		if ( eventType == self.EventKeyDown and theKey == int(InputTypes.KB_M) and self.eventManager.bAlt and gc.getActivePlayer().getNumCities() > 0 and gc.getActivePlayer().getCurrentEra() >= g_iStartingEra):
			self.mercenaryManager.interfaceScreen()


	# This method will remove a mercenary unit from the game if it is killed
	def onUnitKilled(self, argsList):
		'Unit Killed'
		unit, iAttacker = argsList
		
		mercenary = objMercenaryUtils.getMercenary(unit.getNameNoDesc())

		if(mercenary != None and g_bDisplayMercenaryMessages and mercenary.getBuilder() != -1 and unit.isDead()):
			strMessage = mercenary.getName() + " has died under " + gc.getPlayer(mercenary.getOwner()).getName() + "'s service."
			# Inform the player that the mercenary has died.
			CyInterface().addMessage(mercenary.getBuilder(), True, 20, strMessage, "", 0, "", ColorTypes(0), -1, -1, True, True) 

		objMercenaryUtils.removePlayerMercenary(unit)


	# This method will remove a mercenary unit from the game if it is lost
	def onUnitLost(self, argsList):
		'Unit Lost'
		unit = argsList[0]
		
		# Debug code - start
		if(g_bDebug):
			CvUtil.pyPrint("lost: " + unit.getName())
		# Debug code - end
		
		# If the unit being lost is a mercenary, check to see if they have been
		# replaced by an upgraded version of themselves. If they are then save
		# the new upgraded version of themselves and return immediately.
		if(objMercenaryUtils.isMercenary(unit)):

			# Debug code - start
			if(g_bDebug):		
				CvUtil.pyPrint("mercenary unit lost: " + unit.getName())
			# Debug code - end
				
			# Get the active player ID
			iPlayer = gc.getGame().getActivePlayer()
			
			# Get the reference of the actual player
			pyPlayer = PyPlayer(iPlayer)

			# Get the list of units for the player
			unitList = pyPlayer.getUnitList()
				
			# Go through the list of units to see if an upgraded version of 
			# the unit has been added. If it exists then save it and return
			# immediately.
			for unit in unitList:

				if(unit.getUnitType() != argsList[0].getUnitType() and unit.getNameNoDesc() == argsList[0].getNameNoDesc()):

					# Debug code - start
					if(g_bDebug):		
						CvUtil.pyPrint("mercenary unit upgraded: " + unit.getName())
					# Debug code - end
					
					tmpMerc = objMercenaryUtils.createBlankMercenary()
					tmpMerc.loadUnitData(unit)
					tmpMerc.iBuilder = -1
					objMercenaryUtils.saveMercenary(tmpMerc)
					return
					
		mercenary = objMercenaryUtils.getMercenary(unit.getNameNoDesc())

		if(mercenary != None and g_bDisplayMercenaryMessages and mercenary.getBuilder() != -1 and unit.isDead()):
			strMessage = mercenary.getName() + " was lost under " + gc.getPlayer(mercenary.getOwner()).getName() + "'s service."
			# Inform the player that the mercenary has died.
			CyInterface().addMessage(mercenary.getBuilder(), True, 20, strMessage, "", 0, "", ColorTypes(0), -1, -1, True, True) 
		unit = argsList[0]
		
		# Debug code - start
		if(g_bDebug):		
			CvUtil.pyPrint("lost??: " + unit.getNameNoDesc())	
		# Debug code - end

		objMercenaryUtils.removePlayerMercenary(unit)
		

	# This method will redraw the main interface once a unit is promoted. This way the 
	# gold/turn information will be updated.		
	def onUnitPromoted(self, argsList):
		'Unit Promoted'
		pUnit, iPromotion = argsList
		player = PyPlayer(pUnit.getOwner())

		if(objMercenaryUtils.isMercenary(pUnit)):
			CyInterface().setDirty(InterfaceDirtyBits.GameData_DIRTY_BIT, True)		
			
